<?php
/**
 * OOPでBBS (Step6)
 *
 * オブジェクト指向プログラミングで掲示板を作ってみる。
 * ブログでその作り方を晒すために書いたものです。
 * 詳しい事は当該記事を見てください。 
 *
 *
 *           
 * PHP version 5.3
 *
 * LICENSE : このソースはCreative Commons 表示 - 非営利 2.1 日本 Licenseです。
 *           改変は自由ですが営利目的で利用することは出来ません。
 * 
 * @category  Step6
 * @package   OOP-BBS
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 *
 */
 

/**
 * 掲示板をコントロールするクラス
 *
 * ModelとViewの中間に立って仕事を振り分けます。
 * ユーザが起こしたイベントに対して応答し、色々と処理します。
 * 仕分け人なので描画やデータの操作は行いません。
 *
 * <step6>
 * クラスファイル読み込みとインスタンス作成をコンストラクタに移した。
 * それに伴って各インスタンスの変数をプロパティ化。
 * ViewとModelを可変変数で実行するように変更。
 * 
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   Creative Commons BY-NC http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 */

class Controller {
	
	const INIT_FILE = 'bbs.ini';
	
	public function __construct()
	{
	
		include_once('model.php');
		include_once('view.php');
		include_once('bbs_log.php');
		include_once('config.php');
		
		$iniFilePath = dirname($_SERVER['SCRIPT_FILENAME']).'/lib/'.self::INIT_FILE;
	
		$this->config = new Config($iniFilePath);//設定
		$this->model = new Model($this->config);//モデル
		$this->view = new View($this->config);//ビュー
		
	}
    
	/**
	 * index.phpから実行されるメソッド
	 * @return {}
	 * @since 1.0 (step1)
	 */
	public function execute()
	{
	    $this->model->setLogfileName($this->config->get('logfile_name'));//ログファイルをセット
        $this->model->readLog();//ログを読み込む
		
		if(isset($_POST['mode']) && $_POST['mode'] != ''){
			
			//トークンの確認
			$token = $this->model->clean($_POST['token']);
			
			if($token == @$_COOKIE["_token"]){
				$this->view->setSystemMessage('error', '不正な送信です');//エラーをViewに渡す
				
			}else{
				setcookie("_token", $token);
				$action = $this->model->clean($_POST['mode']).'Action';
				$this->$action();
			}
		}
		
		$viewName = 'index';
		
		if(isset($_GET['view']) && $_GET['view'] != ''){
					
			$viewName = $this->model->clean($_GET['view']);
			
		}
		
		$this->view->setTemplateName($viewName);//テンプレートをセット
			
		$viewName = $viewName.'View';
		
		if(method_exists($this, $viewName)){
			$this->view->setToken($this->createToken());//Viewにトークンを渡す
			$this->$viewName();//ビュー実行
		}else{
			die('システムエラーが発生しました');
		}
		
	}
	
	/**
	 * 書き込み
	 */
	function writeAction()
	{
		$this->model->setPostdata($_POST);//送信内容のセット

		if($this->model->validation()===true){
			$this->model->write();//ログに書き込み
			$this->view->setSystemMessage('success', '保存しました');//メッセージをセット
            $this->model->readLog();//再読み込み
		}else{
			$this->view->setSystemMessage('error', $this->model->getError());//エラーをViewに渡す
		}
	}
	
	//削除処理
	function deleteAction()
	{
		var_dump($_POST);
	}
	
	//編集処理
	function editAction()
	{
		var_dump($_POST);
	}
	
	//インデックス
	private function indexView()
	{
		$this->view->setLog($this->model->getLogData());//Viewにログを渡す
		$this->view->display();//表示
	}
	
	//編集画面
	private function editView()
	{
		$log = $this->model->findData($this->model->clean($_GET['id']));
		
		$this->view->setLog($log);//Viewにログを渡す
		$this->view->display();//表示
	}
	
	//削除画面
	private function deleteView()
	{
		$log = $this->model->findData($this->model->clean($_GET['id']));
		
		$this->view->setLog($log);//Viewにログを渡す
		$this->view->display();//表示
	}
	/**
	 * トークンの作成
	 * @return {string} ランダムな半角英数から成る文字列
	 */
	public function createToken()
	{
		return substr(sha1(uniqid().mt_rand()),-13,10);
	}
}
?>